/* FFdecsa -- fast decsa algorithm
 *
 * Copyright (C) 2003-2004  fatih89r
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <mmintrin.h>

#if 0 // some older gcc version need this
#define _m_pand   _mm_and_si64
#define _m_por    _mm_or_si64
#define _m_pxor   _mm_xor_si64
#define _m_psllqi _mm_slli_si64
#define _m_psrlqi _mm_srli_si64
#define _m_empty  _mm_empty
#endif

typedef __m64 group;
#define GROUP_PARALLELISM 64
#define FF0()      ((__m64)0x0ULL)
#define FF1()      ((__m64)0xffffffffffffffffULL)
#define FFAND(a,b) _m_pand((a),(b))
#define FFOR(a,b)  _m_por((a),(b))
#define FFXOR(a,b) _m_pxor((a),(b))
#define FFNOT(a)   _m_pxor((a),FF1())

/* 64 rows of 64 bits */

void static inline FFTABLEIN(unsigned char *tab, int g, unsigned char *data){
#if 1
  *(((int *)tab)+2*g)=*((int *)data);
  *(((int *)tab)+2*g+1)=*(((int *)data)+1);
#else
  *(((long long int *)tab)+g)=*((long long int *)data);
#endif
}

void static inline FFTABLEOUT(unsigned char *data, unsigned char *tab, int g){
#if 1
  *((int *)data)=*(((int *)tab)+2*g);
  *(((int *)data)+1)=*(((int *)tab)+2*g+1);
#else
  *((long long int *)data)=*(((long long int *)tab)+g);
#endif
}

void static inline FFTABLEOUTXORNBY(int n, unsigned char *data, unsigned char *tab, int g){
  int j;
  for(j=0;j<n;j++){
    *(data+j)^=*(tab+8*g+j);
  }
}

typedef __m64 batch;
#define BYTES_PER_BATCH 8
#define B_FFAND(a,b) _m_pand((a),(b))
#define B_FFOR(a,b)  _m_por((a),(b))
#define B_FFXOR(a,b) _m_pxor((a),(b))
#define B_FFN_ALL_29() ((__m64)0x2929292929292929ULL)
#define B_FFN_ALL_02() ((__m64)0x0202020202020202ULL)
#define B_FFN_ALL_04() ((__m64)0x0404040404040404ULL)
#define B_FFN_ALL_10() ((__m64)0x1010101010101010ULL)
#define B_FFN_ALL_40() ((__m64)0x4040404040404040ULL)
#define B_FFN_ALL_80() ((__m64)0x8080808080808080ULL)
#define B_FFSH8L(a,n) _m_psllqi((a),(n))
#define B_FFSH8R(a,n) _m_psrlqi((a),(n))

#define M_EMPTY() _m_empty()


#undef XOR_8_BY
#define XOR_8_BY(d,s1,s2)    do{ __m64 *pd=(__m64 *)(d), *ps1=(__m64 *)(s1), *ps2=(__m64 *)(s2); \
                                 *pd = _m_pxor( *ps1 , *ps2 ); }while(0)

#undef XOREQ_8_BY
#define XOREQ_8_BY(d,s)      do{ __m64 *pd=(__m64 *)(d), *ps=(__m64 *)(s); \
                                 *pd = _m_pxor( *ps, *pd ); }while(0)

#undef COPY_8_BY
#define COPY_8_BY(d,s)       do{ __m64 *pd=(__m64 *)(d), *ps=(__m64 *)(s); \
                                 *pd =  *ps; }while(0)

#undef BEST_SPAN
#define BEST_SPAN            8

#undef XOR_BEST_BY
#define XOR_BEST_BY(d,s1,s2) do{ XOR_8_BY(d,s1,s2); }while(0);

#undef XOREQ_BEST_BY
#define XOREQ_BEST_BY(d,s)   do{ XOREQ_8_BY(d,s); }while(0);

#undef COPY_BEST_BY
#define COPY_BEST_BY(d,s)    do{ COPY_8_BY(d,s); }while(0);
