/*
 * Softcam plugin to VDR (C++)
 *
 * This code is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This code is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 * Or, point your browser to http://www.gnu.org/copyleft/gpl.html
 */

#ifndef ___LOG_H
#define ___LOG_H

// ----------------------------------------------------------------

#define LOPT_NUM    24
#define LOPT_MAX    (1<<LOPT_NUM)
#define LOPT_MASK   (LOPT_MAX-1)
#define LMOD_MAX    (1<<(32-LOPT_NUM))
#define LMOD_MASK   (LMOD_MAX-1)
#define LMOD_ENABLE 1

#define LCLASS(m,o) ((((m)&LMOD_MASK)<<LOPT_NUM)|((o)&LOPT_MASK))
#define LMOD(c)     (((c)>>LOPT_NUM)&LMOD_MASK)
#define LOPT(o)     ((o)&LOPT_MASK)

#define LOG_CON  1
#define LOG_FILE 2
#define LOG_SYS  4

#define ADD_MODULE(MOD,NAME) static bool __add_mod_ ## NAME = cLogging::AddModule(MOD,&NAME);

// ----------------------------------------------------------------

#define L_GEN       0
#define L_GEN_ERROR LCLASS(L_GEN,2)
#define L_GEN_WARN  LCLASS(L_GEN,4)
#define L_GEN_INFO  LCLASS(L_GEN,8)
#define L_GEN_DEBUG LCLASS(L_GEN,16)

// ----------------------------------------------------------------

struct LogModule {
  int OptSupported, OptDefault;
  const char *Name, *OptName[LOPT_NUM];
  };

struct LogHeader;
class cLineBuff;

class cLogging {
private:
  static bool GetHeader(int c, struct LogHeader *lh);
  static void LogLine(const struct LogHeader *lh, const char *txt);
  static const struct LogModule *GetModule(int c);
public:
  static bool AddModule(int m, const struct LogModule *dm);
  //
  static void Printf(int c, const char *format, ...) __attribute__ ((format (printf,2,3)));
  static void Puts(int c, const char *txt);
  static void PutLB(int c, cLineBuff *lb);
  static void Dump(int c, const unsigned char *data, int n, const char *format, ...) __attribute__ ((format (printf,4,5)));
  static bool Enabled(int c);
  //
  static void SetLogMode(int mode);
  static void SetModuleEnable(int c, bool on);
  static void SetModuleOptions(int c, int o);
  static void SetModuleDefault(int c);
  static const char *GetModuleName(int c);
  static const char *GetOptionName(int c);
  static int GetModuleOptions(int c);
  static void ParseConfig(const char *txt);
//  static cString GetConfig(void);
  };

#endif //___LOG_H
